package MT::Plugin::SKR::ElapsedTimer;
# ElapsedTimer - 
#       Copyright (c) 2009 SKYARC System Co.,Ltd.
#       http://www.skyarc.co.jp/engineerblog/entry/elapsedtimer.html

use strict;
use MT 5;
use MT::Entry;
#use Data::Dumper;#DEBUG

use vars qw( $MYNAME $VERSION $SCHEMA_VERSION );
$MYNAME = 'ElapsedTimer';
$VERSION = '0.01 DEVEL';
$SCHEMA_VERSION = '0.0100';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        name => $MYNAME,
        id => lc $MYNAME,
        key => lc $MYNAME,
        version => $VERSION,
        schema_version => $SCHEMA_VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/elapsedtimer.html',
        description => <<HTMLHEREDOC,
<__trans phrase="ElapsedTimer">
HTMLHEREDOC
#        l10n_class => $MYNAME. '::L10N',
});
MT->add_plugin( $plugin );

sub instance { $plugin; }

### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            entry => {
                total_elapsed_time => {
                    type => 'integer',
                    not_null => 0,
                    label => 'Total Elapsed Time',
                },
                elapsed_time => {
                    type => 'integer',
                    not_null => 0,
                    label => 'Elapsed Time',
                    revisioned => 1,
                },
            },
        },
        callbacks => {
            # Add hidden fields
            'MT::App::CMS::template_param.edit_entry' => \&_tmpl_param_edit,
            'MT::App::CMS::template_source.edit_entry' => \&_tmpl_src_edit,
            #
            'cms_pre_save.entry' => \&_hdlr_pre_save,
            'cms_pre_save.page' => \&_hdlr_pre_save,
        },
    });
}



###
sub _tmpl_param_edit {
    my ($cb, $app, $param, $tmpl) = @_;

    $param->{total_elapsed_time} = '0 minute';
    if (!$param->{new_object}) {
        my $entry = MT::Entry->load({ id => $param->{id}});
        if ($entry) {
            my $total_elapsed_time = $entry->total_elapsed_time || 0;
            my $_h = int ($total_elapsed_time / 3600.);
            $param->{total_elapsed_time} = $_h ? "$_h hour " : "";
            my $_m = int ($total_elapsed_time / 60.) % 60;
            $param->{total_elapsed_time} .= "$_m minute";
        }
    }
    $param->{elapsed_start} = time;
}

###
sub _tmpl_src_edit {
    my ($eh, $app, $tmpl) = @_;

    my $old = quotemeta(<<'HTMLHEREDOC');
<mt:if name="new_object">
    <input type="hidden" name="status" id="status" value="1" />
HTMLHEREDOC
    my $new = <<'HTMLHEREDOC';
<p><__trans phrase="Total editing time: [_1]" params="<$mt:var name="total_elapsed_time"$>"></p>
<input type="hidden" name="elapsed_start" value="<$mt:var name="elapsed_start"$>" />
HTMLHEREDOC
    $$tmpl =~ s/($old)/$new$1/;
}



###
sub _hdlr_pre_save {
    my ($cb, $app, $obj) = @_;

    if ($app->isa('MT::App::CMS') && defined(my $q = $app->{query})) {
        if (my $elapsed_start = $q->param('elapsed_start')) {
            $obj->elapsed_time (time - $elapsed_start);
            $obj->total_elapsed_time ($obj->total_elapsed_time + $obj->elapsed_time);
        }
    }
}

1;